/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "winner.h"

void pokerWinnerGetFullHand(poker_t *poker,pokerplayer_t *player,card_t *cards){
  uint8_t i;

  // Add the dealer hand
  for(i = 0; i < poker->cardsFacing; i++) {
    cards[i] = poker->cards[i];
  }

  // Add the player hand
  for(i = 0; i < player->cardCount; i++) {
    cards[i+poker->cardsFacing] = player->cards[i];
  }
}

winning_t pokerWinnerGetStatus(poker_t *poker, pokerplayer_t *player) {
  winning_t winning;
  uint8_t i, j;
  int32_t index;
  card_t card;
  uint8_t number, suit;

  // Get the full poker hand (should be a 7 card hand)
  winning.size = poker->cardsFacing + player->cardCount;
  pokerWinnerGetFullHand(poker, player, winning.full);

  // Reset the winning status.
  winning.count = 0;
  winning.type = 0x00;
  for(i = 0; i < POKER_WINNING_SET_SIZE; i++) winning.set[i] = -1;

  // TESTING HAND
  winning.full[0] = CARD_CLUBS_ACE;
  winning.full[1] = CARD_CLUBS_QUEEN;
  winning.full[2] = CARD_CLUBS_TEN;
  winning.full[3] = CARD_SPADES_FOUR;
  winning.full[4] = CARD_CLUBS_KING;
  winning.full[5] = CARD_DIAMONDS_NINE;
  winning.full[6] = CARD_CLUBS_JACK;
  
  //////////////////////// Now look for the winning set ////////////////////////

  // Ace High Royal flush.
  for(i = 0; i < winning.count; i++) {
    // Look for Ace
    card = winning.full[i];
    number = cardGetNumber(card);
    if(number != CARD_ACE) continue;
    suit = cardGetSuit(card);

    // Now look for the matching cards (Reverse order to order from A to 10)
    for(j = 0; j < 4; j++) {
      index = cardContains(winning.full,winning.size,cardGet(CARD_KING-j,suit));
      if(index == -1) {
        winning.count = 0;
        break;
      }
      winning.set[j+1] = index;
      winning.count++;
    }

    // Check if has all necessary cards. Adds the Ace to the array.
    if(winning.count > 0) {
      winning.set[0] = card;
      winning.count++;
      winning.type = POKER_WINNING_TYPE_ROYAL_FLUSH;
      printf("Royal Flush\n");
      return winning;
    }
  }

  // High card (worst)

  return winning;
}

void pokerWinnerInit(poker_t *poker) {
  uint8_t winners[POKER_PLAYER_COUNT];
  uint8_t winnerCount = 0;

  poker->round = POKER_ROUND_WINNER;
  printf("Winner Winner Chicken Dinner\n");

  // Kill me
  for(uint8_t i = 0; i < POKER_PLAYER_COUNT; i++) {
    pokerplayer_t *player = poker->players + i;
    if(!pokerPlayerIsAlive(player)) continue;

    // Get the players' full hand
    pokerWinnerGetStatus(poker, player);
  }
}